import 'package:flutter/material.dart';

class SlickFlexibleColumn extends StatelessWidget {
  final double minWidthPerChild;
  final List<Widget> children;
  final int maxChildrenPerRow;
  final double maxWidthPerChild;
  final EdgeInsetsGeometry padding;

  final CrossAxisAlignment crossAxisAlignment;

  const SlickFlexibleColumn({
    Key key,
    this.children,
    this.minWidthPerChild = 300,
    this.maxWidthPerChild = 2000,
    this.maxChildrenPerRow = 999,
    this.crossAxisAlignment = CrossAxisAlignment.start,
    this.padding = const EdgeInsets.only(
      right: 8.0,
    ),
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: padding,
      child: LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
          return Column(
            mainAxisSize: MainAxisSize.min,
            crossAxisAlignment: crossAxisAlignment,
            children: getRows(context, constraints.maxWidth),
          );
        },
      ),
    );
  }

  List<Row> getRows(BuildContext context, double maxWidth) {
    var numberPerRow = (maxWidth / minWidthPerChild).floor();
    if (numberPerRow > maxChildrenPerRow) numberPerRow = maxChildrenPerRow;

    var rows = <Row>[];
    var countForRow = 0;
    var currentRowChildren = <Widget>[];
    for (var w in children) {
      if (countForRow == numberPerRow) {
        rows.add(
          Row(
            crossAxisAlignment: CrossAxisAlignment.start,
            mainAxisSize: MainAxisSize.max,
            children: currentRowChildren,
          ),
        );
        currentRowChildren = [];
        countForRow = 0;
      }

      countForRow++;
      currentRowChildren.add(
        Expanded(
          child: w,
        ),
      );
    }

    if (currentRowChildren.isNotEmpty) {
      while (countForRow != numberPerRow) {
        currentRowChildren.add(
          Expanded(
            child: Container(),
          ),
        );
        countForRow++;
      }

      rows.add(
        Row(
          crossAxisAlignment: CrossAxisAlignment.start,
          mainAxisSize: MainAxisSize.max,
          children: currentRowChildren,
        ),
      );
    }

    return rows;
  }
}
