import 'package:flutter/material.dart';
import 'package:slick/controls/form_input/slick_text_input.dart';
import 'package:intl/intl.dart';

enum SlickDateInputMode { Date, Time, DateAndTime }

class SlickDateInput extends StatelessWidget {
  SlickDateInput({
    Key key,
    @required this.initialValue,
    @required this.labelText,
    @required this.onChanged,
    this.inputMode = SlickDateInputMode.Date,
    this.readonly = false,
  }) : super(key: key);

  final DateTime initialValue;
  final String labelText;
  final ValueChanged<DateTime> onChanged;
  final bool readonly;
  final SlickDateInputMode inputMode;

  @override
  Widget build(BuildContext context) {
    return Stack(
      alignment: Alignment.centerRight,
      children: [
        Row(
          children: [
            IconButton(
              icon: Icon(Icons.date_range_rounded, color: Colors.grey),
              onPressed: () async {
                final DateTime picked = await showDatePicker(
                    context: context,
                    initialDate: DateTime.now(),
                    initialDatePickerMode: DatePickerMode.day,
                    firstDate: DateTime(2021),
                    lastDate: DateTime(2030));
                if (picked != null) {}
              },
            ),
            IconButton(
              icon: Icon(Icons.access_time_rounded, color: Colors.grey),
              onPressed: () async {
                final TimeOfDay picked = await showTimePicker(
                  context: context,
                  initialTime: TimeOfDay(
                    hour: DateTime.now().hour,
                    minute: DateTime.now().minute,
                  ),
                );
                if (picked != null) {}
              },
            ),
          ],
        ),
        SlickTextInput(
          readOnly: readonly,
          labelText: labelText,
          initialValue: getInitialValue(),
          onChanged: (val) {
            if (val.isEmpty) onChanged(null);
            var dateVal = DateTime.tryParse(val);
            if (dateVal == null) dateVal = initialValue;
            onChanged(dateVal);
          },
        ),
      ],
    );
  }

  String getInitialValue() {
    if (this.initialValue == null) return '';

    switch (this.inputMode) {
      case SlickDateInputMode.Date:
        return DateFormat('yyyy-MM-dd').format(initialValue);
      case SlickDateInputMode.Time:
        return DateFormat('HH:mm').format(initialValue);
      case SlickDateInputMode.DateAndTime:
        return DateFormat('yyyy-MM-dd HH:mm').format(initialValue);
      default:
        return '';
    }
  }
}
