import 'package:flutter/material.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:slick/configuration/slick_configuration.dart';
import 'package:slick/security/security.dart';
import 'package:slick/slick.dart';
import 'package:slick/state/base_logic.dart';
import 'route_path.dart';

final currentRouteSingleton = CurrentRouteConfiguration();

class CurrentRouteConfiguration {
  String routeName;
  String parameter;
  bool building = false;
  bool reInit = false;
  ValueChanged<RoutePath> _requestRoute;
  RouteInfo routeInformation;
  void navigate(RouteInfo routeInfo, {String parameter}) {
    parameter ??= '';
    _requestRoute(RoutePath.routeInfo(routeInfo, parameter = parameter));
  }

  void navigateHome() {
    _requestRoute(RoutePath.home());
  }

  void updateConfiguration(String routeName, String parameter,
      ValueChanged<RoutePath> requestRoute, bool reInit) {
    this.routeName = routeName;
    this.parameter = parameter;
    this.reInit = reInit;
    _requestRoute = requestRoute;

    if (routeName == null || routeName.isEmpty) {
      routeInformation = SlickConfiguration.dashboardRoute;
    } else {
      var routes = SlickConfiguration.routes
          .where((element) => element.routeName == routeName);
      if (routes.isEmpty) {
        routeInformation = null;
      } else {
        routeInformation = routes.first;
      }
    }
  }

  String screenNameForRoute() {
    for (var info in SlickConfiguration.routes) {
      if (info.routeName == routeName) return info.screenName;
    }
    return '';
  }

  Widget screenForRoute() {
    for (var info in SlickConfiguration.routes) {
      if (info.routeName == routeName) return info.screen();
    }

    return SlickConfiguration.unknownScreen;
  }

  AutoDisposeChangeNotifierProvider<BaseLogic> logicForRoute() {
    for (var info in SlickConfiguration.routes) {
      if (info.routeName == routeName) return info.logicProvider;
    }
    return null;
  }

  bool currentUserHasViewAccess() {
    if (routeInformation == null) return false;
    if (routeInformation.limitToRoles.isEmpty) return true;

    return routeInformation.limitToRoles
            .where((role) => SlickSecurity.currentUser.role.title == role.title)
            .length >
        0;
  }
}

class RouteInfo {
  final String routeName;
  final String screenName;
  final AutoDisposeChangeNotifierProvider<BaseLogic> logicProvider;
  final Function screen;
  final List<SlickRole> limitToRoles;

  RouteInfo({
    @required this.routeName,
    this.screenName = '',
    @required this.logicProvider,
    @required this.screen,
    this.limitToRoles,
  });
}
