import 'package:flutter/material.dart';
import 'route_path.dart';

class SlickRouteInformationParser extends RouteInformationParser<RoutePath> {
  @override
  Future<RoutePath> parseRouteInformation(
      RouteInformation routeInformation) async {
    final uri = Uri.parse(routeInformation.location);

    // Handle '/'
    if (uri.pathSegments.isEmpty) {
      return RoutePath.home();
    }

    String mainParameter;
    String routeName;

    if (uri.pathSegments.isNotEmpty) routeName = uri.pathSegments[0];
    if (uri.pathSegments.length == 2) mainParameter = uri.pathSegments[1];

    return RoutePath.withParameter(routeName, mainParameter);
  }

  @override
  RouteInformation restoreRouteInformation(RoutePath path) {
    if (path.isUnknown) {
      return RouteInformation(location: '/404');
    }
    if (path.isHomePage) {
      return RouteInformation(location: '/');
    }
    if (path.hasParameter) {
      return RouteInformation(location: '/${path.routeName}/${path.parameter}');
    }
    if (path.hasRouteName) {
      return RouteInformation(location: '/${path.routeName}');
    }
    return null;
  }
}
