import 'package:flutter/material.dart';
import 'package:slick/controls/slick_screen.dart';
import 'route_configuration.dart';
import 'route_path.dart';

class SlickRouterDelegate extends RouterDelegate<RoutePath>
    with ChangeNotifier, PopNavigatorRouterDelegateMixin<RoutePath> {
  @override
  final GlobalKey<NavigatorState> navigatorKey;

  String _parameter;
  String _routeName;
  bool show404 = false;

  SlickRouterDelegate() : navigatorKey = GlobalKey<NavigatorState>();

  @override
  RoutePath get currentConfiguration {
    if (show404) {
      return RoutePath.unknown();
    }
    return _parameter == null && _routeName == null
        ? RoutePath.home()
        : RoutePath.withParameter(_routeName, _parameter);
  }

  @override
  Widget build(BuildContext context) {
    return Navigator(
      key: navigatorKey,
      pages: [getPage(context)],
      onPopPage: (route, result) {
        if (!route.didPop(result)) {
          return false;
        }

        _parameter = null;
        _routeName = null;
        show404 = false;
        notifyListeners();

        return true;
      },
    );
  }

  @override
  Future<void> setNewRoutePath(RoutePath path) async {
    if (path.isUnknown) {
      _parameter = null;
      _routeName = null;
      show404 = true;
      return;
    }

    _routeName = path.routeName;
    _parameter = path.parameter;
  }

  void _handleRouteRequest(RoutePath path) {
    _parameter = path.parameter;
    _routeName = path.routeName;
    notifyListeners();
  }

  Page getPage(BuildContext context) {
    var reInit = false;

    if (_routeName == currentRouteSingleton.routeName &&
        _parameter != currentRouteSingleton.parameter) {
      reInit = true;
    }

    currentRouteSingleton.updateConfiguration(
      _routeName,
      _parameter,
      _handleRouteRequest,
      reInit,
    );

    return MaterialPage(child: BaseScreen());
  }
}
